package logging

import (
	"fmt"
	"log"
)

func (l *StdLogger) Setup() {

	l.Logger = log.Default()
	l.Logger.SetPrefix(l.Prefix)
}

func (l *StdLogger) Shutdown() {

	// NOOP
	_ = ""

}

func (l *StdLogger) doDebug(d bool) {
	l.EnableDebug = d
}

func (l *StdLogger) setPrefix(prefix string) {
	l.Prefix = prefix
	l.Logger.SetPrefix(prefix)
}

func (l *StdLogger) GetPrefix() string {
	return l.Prefix
}

func (l *StdLogger) Alert(s string, v ...interface{}) (err error) {

	var msg string

	if v != nil {
		msg = fmt.Sprintf(s, v...)
	} else {
		msg = s
	}

	l.renderWrite(msg, "ALERT")

	return
}

func (l *StdLogger) Crit(s string, v ...interface{}) (err error) {

	var msg string

	if v != nil {
		msg = fmt.Sprintf(s, v...)
	} else {
		msg = s
	}

	l.renderWrite(msg, "CRITICAL")

	return
}

func (l *StdLogger) Debug(s string, v ...interface{}) (err error) {

	if !l.EnableDebug {
		return
	}

	var msg string

	if v != nil {
		msg = fmt.Sprintf(s, v...)
	} else {
		msg = s
	}

	l.renderWrite(msg, "DEBUG")

	return
}

func (l *StdLogger) Emerg(s string, v ...interface{}) (err error) {

	var msg string

	if v != nil {
		msg = fmt.Sprintf(s, v...)
	} else {
		msg = s
	}

	l.renderWrite(msg, "EMERGENCY")

	return
}

func (l *StdLogger) Err(s string, v ...interface{}) (err error) {

	var msg string

	if v != nil {
		msg = fmt.Sprintf(s, v...)
	} else {
		msg = s
	}

	l.renderWrite(msg, "ERROR")

	return
}

func (l *StdLogger) Info(s string, v ...interface{}) (err error) {

	var msg string

	if v != nil {
		msg = fmt.Sprintf(s, v...)
	} else {
		msg = s
	}

	l.renderWrite(msg, "INFO")

	return
}

func (l *StdLogger) Notice(s string, v ...interface{}) (err error) {

	var msg string

	if v != nil {
		msg = fmt.Sprintf(s, v...)
	} else {
		msg = s
	}

	l.renderWrite(msg, "NOTICE")

	return
}

func (l *StdLogger) Warning(s string, v ...interface{}) (err error) {

	var msg string

	if v != nil {
		msg = fmt.Sprintf(s, v...)
	} else {
		msg = s
	}

	l.renderWrite(msg, "WARNING")

	return
}

func (l *StdLogger) renderWrite(msg, prio string) {

	s := fmt.Sprintf("[%v] %v", prio, msg)

	l.Logger.Println(s)

	return
}
